package java;

import java.io.*;
import java.util.ArrayList;

public class MiscTools {

  public MiscTools() {
    m_clusters = new ArrayList(10);
  }

  private void loadMatrix(String fileName, int whichMatrix)
      throws IOException {
    BufferedReader file = new BufferedReader(new FileReader(fileName));
    String line = file.readLine();
    String splits[] = line.trim().split(" ");
    m_total_vertices = splits.length;
    if (whichMatrix == 1) { m_adj_matrix = new double[m_total_vertices][m_total_vertices]; } else if (whichMatrix == 2) {
      m_dist_matrix = new double[m_total_vertices][m_total_vertices];
    }
    int row_count = 1;
    populateMatrix(line, 0, whichMatrix);
    int counter = 0;
    while ((line = file.readLine()) != null) {
      counter++;
      populateMatrix(line, counter, whichMatrix);
      row_count++;
    }
    file.close();
    if (m_total_vertices != row_count) {
      throw new RuntimeException((new StringBuilder()).append("Mismatch in #row,col  : row ").append(row_count).append("  col ").append(m_total_vertices).toString());
    } else { return; }
  }

  private void populateMatrix(String line, int rowIndex, int whichMatrix) {
    String splits[] = line.trim().split(" ");
    for (int i = 0; i < splits.length; i++) {
      double val = Double.parseDouble(splits[i]);
      if (whichMatrix == 1) {
        m_adj_matrix[rowIndex][i] = val;
        continue;
      }
      if (whichMatrix == 2) { m_dist_matrix[rowIndex][i] = val; }
    }

  }

  private void calculateSimMatrix(String adjMatFile)
      throws IOException {
    loadMatrix(adjMatFile, 1);
    m_sim_matrix = new double[m_total_vertices][m_total_vertices];
    int degree_vector[] = new int[m_total_vertices];
    int total_edges = 0;
    for (int i = 0; i < m_total_vertices; i++) {
      int degree = 0;
      for (int j = 0; j < m_total_vertices; j++) {
        int edgeCount = m_adj_matrix[i][j] <= 0.0D ? 0 : 1;
        degree += edgeCount;
        total_edges += edgeCount;
      }

      degree_vector[i] = degree;
    }

    total_edges /= 2;
    m_total_edges = total_edges;
    for (int i = 0; i < m_total_vertices; i++) {
      for (int j = 0; j < m_total_vertices; j++) { m_sim_matrix[i][j] = m_adj_matrix[i][j] - (double) (degree_vector[i] * degree_vector[j]) / (2D * (double) total_edges); }

    }

  }

  public void dumpSimMatrix2File(String adjMatFile, String simMatFile)
      throws IOException {
    calculateSimMatrix(adjMatFile);
    BufferedWriter file = new BufferedWriter(new FileWriter(simMatFile));
    file.write((new StringBuilder()).append(m_total_vertices).append("\n").toString());
    for (int i = 0; i < m_total_vertices; i++) {
      for (int j = 0; j < m_total_vertices; j++) { file.write((new StringBuilder()).append((float) m_sim_matrix[i][j]).append(" ").toString()); }

      file.write("\n");
    }

    file.flush();
    file.close();
  }

  private void readClusters(String clusterFileName)
      throws IOException {
    BufferedReader file = new BufferedReader(new FileReader(clusterFileName));
    String line;
    while ((line = file.readLine()) != null) {
      String splits[] = line.trim().split("\t");
      int length = splits.length;
      ArrayList newCluster = new ArrayList(length);
      for (int i = 0; i < splits.length; i++) {
        int node = Integer.parseInt(splits[i]) - 1;
        newCluster.add(Integer.valueOf(node));
      }

      m_clusters.add(newCluster);
    }
  }

  public double caclModularity(String adjMatFile, String clusterFile)
      throws IOException {
    calculateSimMatrix(adjMatFile);
    readClusters(clusterFile);
    double score = 0.0D;
    for (int cluster_id = 0; cluster_id < m_clusters.size(); cluster_id++) {
      ArrayList a_cluster = (ArrayList) m_clusters.get(cluster_id);
      int cluster_size = a_cluster.size();
      for (int i = 0; i < cluster_size; i++) {
        for (int j = 0; j < cluster_size; j++) { score += m_sim_matrix[((Integer) a_cluster.get(i)).intValue()][((Integer) a_cluster.get(j)).intValue()]; }

      }

    }

    score /= 2D * m_total_edges;
    m_modularity = score;
    return m_modularity;
  }

  public double calcOptFracFromLPOpt(String adjMatFile, double lpOpt)
      throws IOException {
    calculateSimMatrix(adjMatFile);
    double totalPosSim = 0.0D;
    for (int i = 0; i < m_total_vertices; i++) {
      for (int j = 0; j < m_total_vertices; j++) { if (m_sim_matrix[i][j] > 0.0D) { totalPosSim += m_sim_matrix[i][j]; } }

    }

    return (totalPosSim - lpOpt) / (2D * m_total_edges);
  }

  public double calculateOptFracFromDistanceFile(String lpDistFile, String adjMatFile)
      throws IOException {
    loadMatrix(lpDistFile, 2);
    calculateSimMatrix(adjMatFile);
    double post_mat[][] = new double[m_total_vertices][m_total_vertices];
    double neg_mat[][] = new double[m_total_vertices][m_total_vertices];
    double lpopt = 0.0D;
    for (int i = 0; i < m_total_vertices; i++) {
      for (int j = 0; j < m_total_vertices; j++) {
        if (m_sim_matrix[i][j] < 0.0D) {
          neg_mat[i][j] = Math.abs(m_sim_matrix[i][j]);
          post_mat[i][j] = 0.0D;
        } else {
          post_mat[i][j] = m_sim_matrix[i][j];
          neg_mat[i][j] = 0.0D;
        }
      }

    }

    for (int i = 0; i < m_total_vertices; i++) {
      for (int j = 0; j < m_total_vertices; j++) {
        lpopt += post_mat[i][j] * m_dist_matrix[i][j];
        lpopt += neg_mat[i][j] * (1.0D - m_dist_matrix[i][j]);
      }

    }

    return calcOptFracFromLPOpt(adjMatFile, lpopt);
  }

  public double getModularity() {
    return m_modularity;
  }

  public static void main(String args[])
      throws IOException {
    if (args.length < 3) {
      System.out.println("Wrong usage");
      System.out.println("mod <adj_mat_file> <cluster_file>\n\t-Calculates modularity from a cluster");
      System.out.println("sim <adj_mat_file> <op_sim_file\n\t-Creaes s similarity matrix file>");
      System.out.println("fracmod <adj_mat_file> lp opt\n\tCalculates the optimal modularity given the LP optimal");
      System.out.println("fracmod2 <adj_mat_file> <dist_file>\n\t-Calculates the optimal modularity given the LP distance file");
      return;
    }
    if (args[0].equalsIgnoreCase("mod")) {
      MiscTools ob = new MiscTools();
      ob.caclModularity(args[1], args[2]);
      System.out.println((new StringBuilder()).append("Modularity: ").append(ob.getModularity()).toString());
    } else if (args[0].equalsIgnoreCase("sim")) {
      MiscTools ob = new MiscTools();
      ob.dumpSimMatrix2File(args[1], args[2]);
    } else if (args[0].equalsIgnoreCase("fracmod")) {
      MiscTools ob = new MiscTools();
      System.out.println((new StringBuilder()).append("Frac Mod: ").append(ob.calcOptFracFromLPOpt(args[1], Double.parseDouble(args[2]))).toString());
    } else if (args[0].equalsIgnoreCase("fracmod2")) {
      MiscTools ob = new MiscTools();
      System.out.println((new StringBuilder()).append("Frac2 Mod: ").append(ob.calculateOptFracFromDistanceFile(args[2], args[1])).toString());
    } else {
      System.out.println("Wrong usage!");
    }
  }

  private double m_sim_matrix[][];
  private double m_dist_matrix[][];
  private double m_adj_matrix[][];
  private int m_total_vertices;
  private double m_modularity;
  private double m_total_edges;
  ArrayList m_clusters;
}
