% Method to calculate the cluster based on modularity.
% SDP is used to seperate the points and then a random hyperplane is used
% to cut the hyper plane.
%
% Input: Adjacency Matrix
% Output: Two clusters
%

function [BEST_DELTA,OP_CLUSTERS]=sdp_cluster(IP_ADJ_MAT, ORIGINAL_M,ORIGINAL_K, GROUP_TO_SPLIT)
CLUSTER=[];
BEST_DELTA=-99999;
max_tries=5000;
n=length(GROUP_TO_SPLIT);
%Calculate degree vector%

% K=zeros(1,n);
% m=0;
% for i=1:n
%     degree=0;
%     for j=1:n
%         degree=degree+IP_ADJ_MAT(GROUP_TO_SPLIT(i),GROUP_TO_SPLIT(j));
%         m=m + IP_ADJ_MAT(GROUP_TO_SPLIT(i),GROUP_TO_SPLIT(j));
%     end
%     K(i)=degree;
% end
%m=m/2;

%Calculate B Matrix%
B=zeros(n,n);
for i=1:n
    for j=1:n
        %B(i,j)=IP_ADJ_MAT(GROUP_TO_SPLIT(i),GROUP_TO_SPLIT(j))-(K(i)*K(j)/(2*m));
        B(i,j)=IP_ADJ_MAT(GROUP_TO_SPLIT(i),GROUP_TO_SPLIT(j)) - ( (ORIGINAL_K(GROUP_TO_SPLIT(i)) * ORIGINAL_K(GROUP_TO_SPLIT(j))) )/(2*ORIGINAL_M);
    end
end


%Start SDP%
%cvx_clear;
%cvx_begin sdp;
%variable X(n,n) symmetric;
%maximize((1/(4*m)) * trace(B'*X));
%maximize((1/(4*m)) * B(:)'*X(:));
%diag(X)==1;
%X >= 0;
%cvx_end;

%YALMIP%
%X=sdpvar(n,n,'symmetric');
%F=set(diag(X)==1)+set(X>=0);
%solvesdp(F,-((1/(4*m)) * trace(B'*X)));
%X=double(X);

At=sparse(1:n+1:n^2,1:n,ones(n,1),n^2,n,n);  %9x3 Matrix
b=ones(n,1);
c=-B(:);
K=[];
K.s=n;

%[X1,Y,info]=sedumi(At,b,c,K);
%[X1,Y,info]=sdplr(At,b,c,K);
[X1,Y,info]=csdp(At,b,c,K);
X=reshape(X1,n,n); % GA I had earlier taken transpose of this which was a mistake on my part; fortunately X is symmetric which saved a few kicks on my butt!


%Transform the output to Vector Program%
[Q,D]=eig(X);
D1=sqrt(D);
V=(Q*D1)';

clear B
clear X

for tryCount=1:max_tries
    %Cut the Hyper Sphere%
    for rTry=0:10
        R=random('norm',0,1,1,n);
    end

    R=R/norm(R);
    CUT_INT=R*V;
    CUT1=[];
    CUT2=[];
    for i=1:n
        RE=real(CUT_INT(i));
        if (RE>=0)
            CUT1=[CUT1 GROUP_TO_SPLIT(i)];
        else
            CUT2=[CUT2 GROUP_TO_SPLIT(i)];
        end
    end

    %Calculate the DeltaQ associated with this partitioning
    e12=0;
    a1=0;
    a2=0;

    l1=length(CUT1);
    l2=length(CUT2);

    for i=1:l1
        for j=1:l2
            e12=e12+IP_ADJ_MAT(CUT1(i),CUT2(j));
        end
    end
    e12=e12/(2*ORIGINAL_M);

    for i=1:l1
        a1=a1+ORIGINAL_K(CUT1(i));
    end
    a1=a1/(2*ORIGINAL_M);

    for i=1:l2
        a2=a2+ORIGINAL_K(CUT2(i));
    end
    a2=a2/(2*ORIGINAL_M);

    deltaQ=2*(a1*a2 - e12);
    if(BEST_DELTA<deltaQ)
        BEST_DELTA=deltaQ;
        CLUSTER{1}=CUT1;
        CLUSTER{2}=CUT2;
    end
end
%BEST_ITERATION
BEST_DELTA
OP_CLUSTERS=CLUSTER;


